package soa.sca.samples.candystore.ejb3;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import soa.sca.samples.candystore.Entry;
import soa.sca.samples.candystore.Item;
import soa.sca.samples.candystore.NotFoundException;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

@Stateless
public class StatelessCartBean implements RemoteCart{

	@PersistenceContext(unitName="Cart")
	private EntityManager em;
	
	public String getTotal() {
        List<Entry<String, Item>> entries = getAll(); 
        double total = 0;
        String currencySymbol = "";
        if (entries.size() > 0) {
            Item item = entries.get(0).getData();
            currencySymbol = item.getPrice().substring(0, 1);
        }
        for (Entry<String, Item> entry : entries) {
            Item item = entry.getData();
            total += Double.valueOf(item.getPrice().substring(1));
        }
        return currencySymbol + total;
	}

	public void delete(String key) throws NotFoundException {
        if (key == null || key.equals("")) {
            Query query = em.createQuery("delete from CartItem");
            query.executeUpdate();
        } else {
            Query query = em.createQuery("delete from CartItem c where c.id=:id").setParameter("id",key);
            int count = query.executeUpdate();
            if (count == 0)
                throw new NotFoundException(key, -2);
        }

        em.flush();
        em.clear();
	}

	public Item get(String key) throws NotFoundException {
		CartItem cartItem = em.find(CartItem.class, key);
		if (cartItem!= null)
		{
			Item item = new Item(cartItem.getName(),cartItem.getPrice());
			return item;
		}else
            throw new NotFoundException(key, -2);
	}

	public List<Entry<String, Item>> getAll() {
        Query query = em.createQuery("select c from CartItem c");
        List<CartItem> cartItems = query.getResultList();
        List<Entry<String, Item>> entries = new ArrayList<Entry<String, Item>>();
        for (int i=0; i<cartItems.size(); i++) {
            Item item = new Item(cartItems.get(i).getName(), cartItems.get(i).getPrice());
            entries.add(new Entry<String, Item>(cartItems.get(i).getId(), item));
        }
        return entries;
	}

	public String post(String key, Item item) {
        if (key == null) {
            key = "cart-" + UUID.randomUUID().toString();
        }
		CartItem cartItem = new CartItem(key,item.getName(),item.getPrice());
		em.persist(cartItem);
        em.flush();
        em.clear();
		return cartItem.getId();
	}

	public void put(String key, Item item) throws NotFoundException {
		CartItem cartItem = em.find(CartItem.class, key);
		if (cartItem !=null)
		{
			cartItem.setName(item.getName());
			cartItem.setPrice(item.getPrice());
			em.persist(cartItem);
		}else
            throw new NotFoundException(key, -2);
        em.flush();
        em.clear();
		
	}

	public List<Entry<String, Item>> query(String queryString) {
        Query query = em.createNativeQuery("select * from cart where " +queryString);
        List<CartItem> cartItems = query.getResultList();
        List<Entry<String, Item>> entries = new ArrayList<Entry<String, Item>>();
        for (int i=0; i<cartItems.size(); i++) {
            Item item = new Item(cartItems.get(i).getName(), cartItems.get(i).getPrice());
            entries.add(new Entry<String, Item>(cartItems.get(i).getId(), item));
        }
        return entries;
	}
	
	

}
