/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package soa.sca.samples.candystore;

import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.UUID;

import org.osoa.sca.annotations.Init;
import org.osoa.sca.annotations.Scope;
import org.osoa.sca.annotations.Service;


@Scope("COMPOSITE")
@Service(interfaces={Cart.class, Total.class})
public class ShoppingCartImpl implements Cart, Total {
    
    private Map<String, Item> cart;
    
    @Init
    public void init() {
        cart = new HashMap<String, Item>();
    }

    public List<Entry<String, Item>> getAll() {
        List<Entry<String, Item>> entries = new ArrayList<Entry<String, Item>>();
        int i = 0;
        for (Map.Entry<String, Item> e: cart.entrySet()) {
        	entries.add(i++,new Entry<String, Item>(e.getKey(), e.getValue()));
            //entries[i++] = new Entry<String, Item>(e.getKey(), e.getValue());
        }
        return entries;
    }

    public Item get(String key) throws NotFoundException {
        Item item = cart.get(key);
        if (item == null) {
            throw new NotFoundException(key, 1);
        } else {
            return item;
        }
    }

    public String post(String key, Item item) {
        if (key == null) {
            key ="cart-" + UUID.randomUUID().toString();
        }
        cart.put(key, item);
        return key;
    }

    public void put(String key, Item item) throws NotFoundException {
        if (!cart.containsKey(key)) {
            throw new NotFoundException(key, 1);
        }
        cart.put(key, item);
    }
    
    public void delete(String key) throws NotFoundException {
        if (key == null || key.equals("")) {
            cart.clear();
        } else {
            Item item = cart.remove(key);
            if (item == null)
                throw new NotFoundException(key, 1);
        }
    }

    public List<Entry<String, Item>> query(String queryString) {
        List<Entry<String, Item>> entries = new ArrayList<Entry<String,Item>>();
        if (queryString.startsWith("name=")) {
            String name = queryString.substring(5);
            for (Map.Entry<String, Item> e: cart.entrySet()) {
                Item item = e.getValue();
                if (item.getName().equals(name)) {
                    entries.add(new Entry<String, Item>(e.getKey(), e.getValue()));
                }
            }
        }
        return entries;
    }
    
    public String getTotal() {
        double total = 0;
        String currencySymbol = "";
        if (!cart.isEmpty()) {
            Item item = cart.values().iterator().next();
            currencySymbol = item.getPrice().substring(0, 1);
        }
        for (Item item : cart.values()) {
            total += Double.valueOf(item.getPrice().substring(1));
        }
        
        
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMinimumFractionDigits(2);
        nf.setMaximumFractionDigits(2);
        String balanceString = nf.format(total);
        
        return currencySymbol + String.valueOf(balanceString);
    }

}
