/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package soa.sca.samples.candystore.db;


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.UUID;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.osoa.sca.ServiceRuntimeException;
import org.osoa.sca.annotations.Destroy;
import org.osoa.sca.annotations.Init;
import org.osoa.sca.annotations.Property;
import org.osoa.sca.annotations.Scope;
import org.osoa.sca.annotations.Service;

import soa.sca.samples.candystore.Cart;
import soa.sca.samples.candystore.Entry;
import soa.sca.samples.candystore.Item;
import soa.sca.samples.candystore.NotFoundException;
import soa.sca.samples.candystore.Total;

@Scope("COMPOSITE")
@Service(interfaces={Cart.class, Total.class})
public class ShoppingCartTableImpl implements Cart, Total {
    
    @Property
    public String database;
    
    private DataSource ds;
    
    private String url; 

    @Init
    public void init() {
        initDataSource(database);
    }
    
    @Destroy
    public void shutdown() {
    }

    public List<Entry<String, Item>> getAll() {
        return getAllInternal();
    }

    public Item get(String key) throws NotFoundException {
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            ResultSet results = statement.executeQuery("select * from Cart where id = '" + key + "'");
            connection.close();
            
            if (results.next()) {
                String name = results.getString("name");
                String price = results.getString("price");
                connection.close();
                return new Item(name, price);
            } else {
            	connection.close();
                throw new NotFoundException(key, -2);
            }
            
           
        } catch (SQLException e) {
            throw new ServiceRuntimeException(e);
        } 
    }

    public String post(String key, Item item) {
        if (key == null) {
            key = "cart-" + UUID.randomUUID().toString();
        }
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            String query = "insert into Cart values ('" + key + "', '" + item.getName() + "', '" + item.getPrice() + "')";
            System.out.println(query);
            statement.executeUpdate(query);
            connection.close();
        } catch (SQLException e) {
            throw new ServiceRuntimeException(e);
        }
        return key;
    }

    public void put(String key, Item item) throws NotFoundException {
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            String query = "update into Cart set name = '" + item.getName() + "', price = '" + item.getPrice() + "' where id = '" + key + "'";
            System.out.println(query);
            int count = statement.executeUpdate(query);
            if (count == 0)
                throw new NotFoundException(key, -2);
            connection.close();
        } catch (SQLException e) {
            throw new ServiceRuntimeException(e);
        }
    }
    
    public void delete(String key) throws NotFoundException {
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            if (key == null || key.equals("")) {
                String query = "delete from Cart";
                System.out.println(query);
                statement.executeUpdate(query);
            } else {
                String query = "delete from Cart where id = '" + key + "'";
                System.out.println(query);
                int count = statement.executeUpdate(query);
                if (count == 0)
                    throw new NotFoundException(key, -2);
            }
            connection.close();
        } catch (SQLException e) {
            throw new ServiceRuntimeException(e);
        }
    }

    public List<Entry<String, Item>> query(String queryString) {
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            ResultSet results = statement.executeQuery("select * from Cart where " + queryString);
            List<Entry<String, Item>> entries = new ArrayList<Entry<String, Item>>();
            while (results.next()) {
                Item item = new Item(results.getString("name"), results.getString("price"));
                entries.add(new Entry<String, Item>(results.getString("id"), item));
            }
            connection.close();
            return entries;
        } catch (SQLException e) {
            throw new ServiceRuntimeException(e);
        }
    }
    
    public String getTotal() {
        List<Entry<String, Item>> entries = getAllInternal(); 
        double total = 0;
        String currencySymbol = "";
        if (entries.size() > 0) {
            Item item = entries.get(0).getData();
            currencySymbol = item.getPrice().substring(0, 1);
        }
        for (Entry<String, Item> entry : entries) {
            Item item = entry.getData();
            total += Double.valueOf(item.getPrice().substring(1));
        }
                
        NumberFormat nf = NumberFormat.getInstance();
        nf.setMinimumFractionDigits(2);
        nf.setMaximumFractionDigits(2);
        String balanceString = nf.format(total);
        
        return currencySymbol + String.valueOf(balanceString);
    }
    
    /*
     * Factored into its own method since the same logic is used by multiple method impls.
     */
    private List<Entry<String, Item>> getAllInternal() {
        try {
            Connection connection = getConnection();
            Statement statement = connection.createStatement();
            ResultSet results = statement.executeQuery("select * from Cart");
            List<Entry<String, Item>> entries = new ArrayList<Entry<String, Item>>();
            while (results.next()) {
                Item item = new Item(results.getString("name"), results.getString("price"));
                entries.add(new Entry<String, Item>(results.getString("id"), item));
            }
            connection.close();
            return entries;
        } catch (SQLException e) {
            throw new ServiceRuntimeException(e);
        }
    }

    private Connection getConnection() throws SQLException {
        Connection connection = null;
        try {
            connection = ds.getConnection();
            if (connection == null) {
                throw new RuntimeException(
                        "Could not obtain a Connection from DataSource");
            }
            connection.setAutoCommit(false);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return connection;
    }

    private void initDataSource(String database) {
        InitialContext ctx;
        try {
            ctx = new InitialContext();
        } catch (NamingException e) {
            throw new RuntimeException(e);
        }
        try {
            this.ds = (DataSource) ctx.lookup(database);
        } catch (NamingException e) {
            throw new RuntimeException(e);
        }
    }
    
}
