/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */

package soa.sca.samples.candystore.merger;

import java.security.Principal;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.security.auth.Subject;

import org.osoa.sca.RequestContext;
import org.osoa.sca.annotations.Context;
import org.osoa.sca.annotations.Property;
import org.osoa.sca.annotations.Reference;
import org.osoa.sca.annotations.Scope;
import org.osoa.sca.annotations.Service;

import soa.sca.samples.candystore.Catalog;
import soa.sca.samples.candystore.CurrencyConverter;
import soa.sca.samples.candystore.Item;


@Service(Catalog.class)
@Scope("COMPOSITE")
public class MergedCatalogImpl implements Catalog {

    @Property
    public String currencyCode = "USD";
    
    @Reference
    public CurrencyConverter currencyConverter;
    
    @Reference
    public Catalog candyCatalog;
    
    @Reference
    public Catalog fruitsCatalog;
    
	@Context
    protected RequestContext requestContext;
    
    public List<Item> get() {
    	
    	this.getSubject();
    	
        String currencySymbol = currencyConverter.getCurrencySymbol(currencyCode);
        
        List<Item> candy = candyCatalog.get();
        List<Item> fruits = fruitsCatalog.get();
        
        List<Item> catalog = new ArrayList<Item>(candy.size() + fruits.size());
        int i =0;
        for (Item item: candy) {
            double price = Double.valueOf(item.getPrice().substring(1));  
            price = currencyConverter.getConversion("USD", currencyCode, price);
            catalog.add(i++, new Item(item.getName(), currencySymbol + price));
            //catalog[i++] = new Item(item.getName(), currencySymbol + price);
        }
        
        for (Item item: fruits) {
            double price = Double.valueOf(item.getPrice().substring(1));  
            price = currencyConverter.getConversion("USD", currencyCode, price);
            catalog.add(i++, new Item(item.getName(), currencySymbol + price));
            //catalog[i++] = new Item(item.getName(), currencySymbol + price);
            
        }
        
        return catalog;
    }
    
    
	private void getSubject() {
		StringBuffer errorMsg = new StringBuffer();
		
        try {
            if (requestContext == null) {
                if (errorMsg == null) {
                    errorMsg = new StringBuffer();
                }
                errorMsg.append("** FAIL: RequestContext is null ");
            }
            Subject subject = null;
            if (requestContext != null) {
                subject = requestContext.getSecuritySubject();
            }
            if (subject == null) {
                if (errorMsg == null) {
                    errorMsg = new StringBuffer();
                }
                errorMsg.append("** FAIL: Subject is null ");
            }
            Principal principal = null;
            if (subject != null) {
                java.util.Set principalSet = subject.getPrincipals();
                if (principalSet.size() > 0)
                {
                    Iterator principalIterator = principalSet.iterator();
                    if (principalIterator.hasNext())
                    {
                        principal = (java.security.Principal) principalIterator.next();
                    }
                }

            }
            if (principal == null) {
                if (errorMsg == null) {
                    errorMsg = new StringBuffer();
                }
                errorMsg.append(" ** FAIL: Principal is null ");
            }
            
            System.out.println("CandyStore: MergedCatalogImpl.getSubject=" + principal.getName());
            
            
        } catch (Exception ex) {
            if (errorMsg == null) {
                errorMsg = new StringBuffer();
            }
            errorMsg.append(" ** FAIL: Caught Exception " + ex.getMessage());
            ex.printStackTrace();
        }
	}

}
