package soa.sca.samples.multiservice;


import javax.jws.WebService;

import org.osoa.sca.annotations.Service;
import org.osoa.sca.annotations.Scope;
import org.osoa.sca.annotations.Reference;

import com.ibm.websphere.samples.webservices.stock.StockQuote;
import helloworld.HelloWorldRemotable;
import com.app.resumebank.ResumeBank;
import com.ibm.websphere.ejb3sample.counter.RemoteCounter;

@Service(MultiService.class)
@Scope("Stateless")
public class MultiServiceImpl implements MultiService {

	@Reference
	public StockQuote stockQuoteService;
	
	@Reference
	public HelloWorldRemotable helloWorldService;
	
	@Reference
	public SecureHelloWorldService secureHelloWorldService;
	
    private ResumeBank extEJBService = null;
    
    private RemoteCounter ejbCounterService = null;

    
    public MultiServiceImpl() {

    }
    
    public ResumeBank getExtEJBService() {
        return extEJBService;
    }
    
    public RemoteCounter getEjbCounterService() {
        return ejbCounterService;
    }

    @Reference
    public void setExtEJBService(ResumeBank extEJBService) {
        this.extEJBService = extEJBService;
    }
    
    @Reference
    public void setEjbCounterService(RemoteCounter ejbCounterService) {
        this.ejbCounterService = ejbCounterService;
    }
    
    
    /**
     * 
     * @param input A stock symbol such as IBM
     * @return The current price of the stock as a string.
     */
	private String quote(String input) throws SCAServiceException {

		String response = "ERROR";


			float quote = stockQuoteService.getQuote(input);
			
			//The quote service returns a price of zero when and invalid stock symbol is entered.
			//We want to throw a business exception in this case.
			if (quote == 0.0) {

				System.out.println(">> ERROR: Stock quote business exception .... invalid symbol: " + input);
				throw new SCAServiceException("The symbol " + input + " is not a valid stock symbol.", 1);
				
			}
			
			response = Float.toString(quote);	
			
			return response;

	    
	}
	
	/**
	 * 
	 * @param name An input string such as a name.
	 * @return Returns the input string with an appended suffix.
	 */
	private String helloWorld(String name){
		String response = "Error";
		
		try {

			helloWorldService.getHello(name);
            // Sleep for 3 seconds to wait the callback to happen
            Thread.sleep(3000);
            response = helloWorldService.getmsg();
            
            return response;

		} catch (Exception e) {
			System.out.println(">> ERROR: hello world EXCEPTION.");
			e.printStackTrace();
			return response + ">>> Hellow World SERVICE EXCEPTION<<< ";
		}
		
		
	}
	
	/**
	 * 
	 * @param name An input string such as a name.
	 * @return Returns the input string with an appended suffix.
	 */
	private String secureHelloWorld(String name){
		String response = "Error";
		
			response = secureHelloWorldService.secureHelloWorld(name);
            
            return response;
		
	}
	
	
	/**
	 * 
	 * @param name An input string such as a name.
	 * @return Returns a confirmation number
	 */
	private String postResume(String name){
		String response = "Error";
		
		try {
			
			response = extEJBService.postResume(name);
            return response;
            
		} catch (Exception e) {
			System.out.println(">> ERROR: Post Resume EXCEPTION.");
			e.printStackTrace();
			return response + ">>> Post Resume SERVICE EXCEPTION<<< ";
		}
		
		
	}
	

	

  /**
   * Increments the ejb counter and returns the current value
   *
   * @return The current value as a String
   */
	private String ejbCounter() {
		
		return Integer.toString(ejbCounterService.increment());
		
	}

    @Override
    public MultiInfo getInfo(String name, String symbol) throws SCAServiceException{
		String stockPrice;
		String helloWorld;
		String secureHelloWorld;
		String confirmationNum;
		String serviceCount;
	
		stockPrice = this.quote(symbol);
		helloWorld = this.helloWorld(name);
		secureHelloWorld = this.secureHelloWorld(name); 
		confirmationNum = this.postResume(name);
		serviceCount = this.ejbCounter();
	

		return new MultiInfo(stockPrice, helloWorld, serviceCount, secureHelloWorld, confirmationNum);
    }




}