package soa.sca.samples.multiservice;

import java.security.Principal;
import java.util.Iterator;

import javax.security.auth.Subject;

import helloworld.HelloWorldRemotable;

import org.osoa.sca.RequestContext;
import org.osoa.sca.annotations.Context;
import org.osoa.sca.annotations.Reference;
import org.osoa.sca.annotations.Service;


@Service(SecureHelloWorldService.class)
public class SecureHelloWorldServiceImpl implements SecureHelloWorldService{
    
	@Context
    protected RequestContext requestContext;
	
	@Reference
	public HelloWorldRemotable helloWorldService;

	public String secureHelloWorld(String name){
		
		String subject = this.getSubject();
		helloWorldService.getHello(name);
		try {
		   Thread.sleep(2000);
		}catch (Exception e) {
			System.out.println(">> ERROR: secure hello world EXCEPTION.");
			e.printStackTrace();
			return ">>> Secure Hello World SERVICE EXCEPTION<<< in SecureHelloWorldServiceImpl.secureHelloWorld(String)";
		}

		return helloWorldService.getmsg() + " You are logged in as: " + this.getSubject();			
	
	}
	
	private String getSubject() {
		StringBuffer errorMsg = new StringBuffer();
		
        try {
            if (requestContext == null) {
                if (errorMsg == null) {
                    errorMsg = new StringBuffer();
                }
                errorMsg.append("** FAIL: RequestContext is null ");
            }
            Subject subject = null;
            if (requestContext != null) {
                subject = requestContext.getSecuritySubject();
            }
            if (subject == null) {
                if (errorMsg == null) {
                    errorMsg = new StringBuffer();
                }
                errorMsg.append("** FAIL: Subject is null ");
            }
            Principal principal = null;
            if (subject != null) {
                java.util.Set principalSet = subject.getPrincipals();
                if (principalSet.size() > 0)
                {
                    Iterator principalIterator = principalSet.iterator();
                    if (principalIterator.hasNext())
                    {
                        principal = (java.security.Principal) principalIterator.next();
                    }
                }
                if (principal != null) {
                    //local.append("security principal = ");
                    //String princname = principal.getName();
                    //String [] username = princname.split("/");
                    //local.append(username[1]);
                }
            }
            if (principal == null) {
                if (errorMsg == null) {
                    errorMsg = new StringBuffer();
                }
                errorMsg.append(" ** FAIL: Principal is null ");
            }
            
            System.out.println("principal.getName()=" + principal.getName());
            return principal.getName();
            
        } catch (Exception ex) {
            if (errorMsg == null) {
                errorMsg = new StringBuffer();
            }
            errorMsg.append(" ** FAIL: Caught Exception " + ex.getMessage());
            ex.printStackTrace();
            return "Error from SecureHelloWorldServiceImpl.getSubject()";
        }
	}

}
