package samples.mybank;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.osoa.sca.annotations.Init;
import org.osoa.sca.annotations.Service;

import com.mybank.account.AccountReport;
import com.mybank.account.AccountService;
import com.mybank.account.AccountSummary;
import com.mybank.account.ObjectFactory;

@Service(AccountService.class)
public class AccountServiceImpl implements AccountService {

    private DataSource ds;

    @Init
    public void init() {
        initDataSource("mybank");
    }

   public com.mybank.account.AccountReport getAccountReport(int param0)  {
      final ObjectFactory objectFactory = new ObjectFactory();
      AccountReport accountReport = objectFactory.createAccountReport();
      accountReport.getAccountSummaries().addAll(getAccountSummaries(param0));
      return accountReport;
   }

   private List<AccountSummary> getAccountSummaries(int param0) {
       final ObjectFactory objectFactory = new ObjectFactory();
       ArrayList<AccountSummary> accountSummaries = new ArrayList<AccountSummary>();
        try {
            Connection conn = getConnection();
            PreparedStatement ps = conn.prepareStatement(
                            "SELECT accountNumber, accountID, accountType, balance FROM accounts where accountNumber = ?");
            ps.setInt(1, param0);
            ps.execute();
            ResultSet rs = ps.getResultSet();
            
            while ( rs.next() ) {
                AccountSummary summary = objectFactory.createAccountSummary();
                summary.setAccountNumber(rs.getInt(1));
                summary.setAccountID(rs.getString(2));
                summary.setAccountType(rs.getString(3));
                summary.setBalance(rs.getFloat(4));
                
                accountSummaries.add(summary);
            }
            conn.close();
        } catch (SQLException ex) {
            throw new RuntimeException(ex);
        }

        return accountSummaries;
    }


    private Connection getConnection() throws SQLException {
        Connection connection = null;
        try {
            connection = ds.getConnection();
            if (connection == null) {
                throw new RuntimeException(
                        "Could not obtain a Connection from DataSource");
            }
            connection.setAutoCommit(false);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return connection;
    }

    private void initDataSource(String database) {
        InitialContext ctx;
        try {
            ctx = new InitialContext();
        } catch (NamingException e) {
            throw new RuntimeException(e);
        }
        try {
            this.ds = (DataSource) ctx.lookup(database);
        } catch (NamingException e) {
            throw new RuntimeException(e);
        }
    }


}


