/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.    
 */
package helloworld.impl;

import helloworld.HelloWorldCallback;
import helloworld.HelloWorldRemotable;
import helloworld.AsynchTranslatorService;

import org.osoa.sca.annotations.Reference;
import org.osoa.sca.annotations.Scope;
import org.osoa.sca.annotations.Service;

/**
 * This client component 
 */
@Scope("COMPOSITE")
@Service(HelloWorldRemotable.class)
public class HelloWorldServiceComponent implements HelloWorldRemotable, HelloWorldCallback {
   
    private String msg = "callback failed";
    private AsynchTranslatorService asynchTranslatorService;

    @Reference
    public void setAsynchTranslatorService(AsynchTranslatorService asynchTranslatorService) {
        this.asynchTranslatorService = asynchTranslatorService;
    }

    public void getHello(String name) {
        asynchTranslatorService.getGreetings(name);
    }

    public void getGreetingsCallback(String getGreetingsReturn) {
        System.out.println("Callback: " + getGreetingsReturn);
        msg = "Callback: " + getGreetingsReturn;
    }
   
    public String getmsg() {
        return msg;
    }
    
}
